@@ -22,6 +22,8 @@ import pprint
 from ast import literal_eval
 from _thread import get_ident as _get_ident
 
+from typing import Mapping, Optional, Sequence, Union
+
 # external imports
 import numpy as np
 
@@ -45,8 +47,7 @@ ON_WINDOWS = platform.system() == "Windows"
 
 
 def case_preserving_replace(string, old, new):
-    """ Replace string while preserving the case
-    """
+    """Replace string while preserving the case"""
 
     def repl(match):
         current = match.group()
@@ -379,6 +380,57 @@ class AnyPyProcessOutputList(collections.abc.MutableSequence):
             for elem in self
         ]
 
+    def to_dataframe(self, index_var="auto", group_var=None):
+        """Return output of all simuations as a concatenated pandas dataframe.
+
+        Parameters:
+        -----------
+        index_var: str
+            Name of the variable to use as axis 0 in the dataframe.
+            If not given system will look for variables ending with
+            "Ouput.Abcsissa.t"
+        group_var: str
+            Name of the variable which will be different across all
+            simulations. If not specified the index of the simuation will
+            be used, and a categorical 'group' column will be added to the
+            dataframe.
+
+        Returns:
+        --------
+        pandas.DataFrame
+        """
+        try:
+            import pandas as pd
+        except ImportError:
+            raise ImportError("pandas is required for this function")
+
+        dfs = []
+        index_name = None
+        for idx, elem in enumerate(self):
+            df = elem.to_dataframe(index_var)
+
+            if index_name and df.index.name != index_name:
+                raise ValueError(
+                    "The index of the dataframe is not consistant across all elements of the output. "
+                )
+            else:
+                index_name = df.index.name
+
+            if group_var is not None:
+                group = group_var
+                if group_var not in df.columns:
+                    raise KeyError(
+                        f"The group variable {group_var} is not available element {idx}"
+                    )
+            else:
+                group = "group"
+                df[group] = idx
+            dfs.append(df)
+        df_out = pd.concat(dfs, ignore_index=True, sort=False)
+        df_out[group] = pd.Categorical(df_out[group])
+
+        return df_out
+
 
 def _expand_short_path_name(short_path_name):
     from ctypes import create_unicode_buffer, windll
@@ -550,12 +602,97 @@ class AnyPyProcessOutput(collections.OrderedDict):
         finally:
             del _repr_running[call_key]
 
+    def to_dataframe(self, index_var: Optional[str] = "auto"):
+        """Convert the output to a pandas dataframe.
+
+        Parameters:
+        -----------
+        index_var: str
+            Name of the variable to use as axis 0 in the dataframe.
+            If "auto" is given the system will look for variables ending with
+            "Ouput.Abcsissa.t". If 'None' no index variable is used an only a single
+            row is returned in the dataframe.
+
+        Returns:
+        --------
+        df: pandas.DataFrame
+            Dataframe with the output data.
+        """
+        try:
+            import pandas as pd
+        except ImportError:
+            raise ImportError("pandas is required for this function")
+
+        excluded_vars = ["task_macro"]
+
+        var_list = set(self.keys()) - set(excluded_vars)
+
+        if index_var == "auto":
+            timevars = [var for var in self if var.endswith("Output.Abscissa.t")]
+            if len(timevars) > 1:
+                raise ValueError(
+                    f"Multiple time variables found. Use 'index_var' argument to indicate which to use."
+                )
+            if len(timevars) == 0:
+                raise ValueError(
+                    "No time variable found. Use 'index_var' argument to indicate what variable should be used."
+                )
+            index_var = timevars[0]
+
+        if index_var is not None:
+            if index_var not in self:
+                raise ValueError(
+                    f"The index var {index_var} could not be found in the data "
+                )
+            index_data = np.array(self[index_var])
+            if index_data.ndim != 1:
+                raise ValueError(f"The index var {index_var} should be a 1D array")
+
+            index_len = index_data.shape[0]
+            df_output = pd.DataFrame({index_var: self[index_var]})
+            # columns = [abscissa]
+            var_list -= set([index_var])
+        else:
+            index_len = 1
+            df_output = pd.DataFrame()
+            # columns = []
+
+        for var in var_list:
+            data = self[var]
+            # col_names = [var]
+            if isinstance(data, (int, float, str)):
+                data = np.array(data)
+            if isinstance(data, np.ndarray):
+                indices = np.array(list(np.ndindex(data.shape)))
+                data = np.atleast_2d(data.T).T
+                # if np.issubdtype(data.dtype, np.number):
+                if data.shape[0] != index_len:
+                    data = data.flatten()
+                    data = np.repeat(data[np.newaxis, :], index_len, axis=0)
+                else:
+                    indices = np.array(list(np.ndindex(data.shape[1:])))
+
+                if len(indices) == 1:
+                    col_names = [var]
+                else:
+                    col_names = [
+                        var + "".join(f"[{i}]" for i in index) for index in indices
+                    ]
+                df_output = pd.concat(
+                    [df_output, pd.DataFrame(data, columns=col_names)], axis=1
+                )
+
+        df_output = df_output.convert_dtypes()
+        if index_var:
+            df_output.set_index(index_var, inplace=True)
+
+        return df_output
 
 
 def _recursive_replace(iterable, old, new):
     for i, elem in enumerate(iterable):
         if isinstance(elem, list):
-            recursive_replace(elem, old, new)
+            _recursive_replace(elem, old, new)
         elif elem == old:
             iterable[i] = new
 
@@ -577,7 +714,7 @@ def _parse_data(val):
                 # handle the case where AnyBody has output 'nan' values
                 val2 = val.replace("nan,", ' "nan",')
                 out = literal_eval(val2)
-                recursive_replace(out, "nan", float("nan") )
+                recursive_replace(out, "nan", float("nan"))
             else:
                 raise SyntaxError
         except (SyntaxError, ValueError):
@@ -592,7 +729,6 @@ def _parse_data(val):
     return out
 
 
-
 ABOVE_NORMAL_PRIORITY_CLASS = 0x8000
 BELOW_NORMAL_PRIORITY_CLASS = 0x4000
 IDLE_PRIORITY_CLASS = 0x0040
@@ -603,7 +739,7 @@ NAME_PATTERN = re.compile(r"Main\.[\w\.]*")
 
 
 def correct_dump_prefix(raw, idx):
-    """ Find the correct prefix to use in the output """
+    """Find the correct prefix to use in the output"""
     # The -1000 hack is to avoid coping large strings in memory,
     # since we really only need to access the previous line.
     dumpline = raw[max(0, idx - 1000) : idx].strip().rsplit("\n", 1)[-1]
@@ -623,9 +759,9 @@ DUMP_PATTERN = re.compile(r"^(Main.*?)\s=\s(.*?(?:\n\s\s.*?)*);", flags=re.M)
 def parse_anybodycon_output(
     raw, errors_to_ignore=None, warnings_to_include=None, fatal_warnings=False
 ):
-    """ Parse the output log file from AnyBodyConsole to
-        for data, errors and warnings. If fatal_warnins is
-        True, then warnings are also added to the error list.
+    """Parse the output log file from AnyBodyConsole to
+    for data, errors and warnings. If fatal_warnins is
+    True, then warnings are also added to the error list.
     """
     warnings_to_include = warnings_to_include or []
     errors_to_ignore = errors_to_ignore or []
